#!/usr/bin/perl -w

use strict;

use Getopt::Long;
use Pod::Usage;
use File::Basename;
#use Data::Dump qw {dump};

use strict;

BEGIN {
  our $root=(fileparse($0))[1];
  $root=$ENV{MURASAKI} if $ENV{MURASAKI};
  unshift(@INC,$root);
  require "$root/config.pl";
  require "$root/common.pl";
  unshift(@INC,$root);
}
use KOG;

my ($man,$help,$makecds,$seqfile);
my (@seqnames,$output_opt);
my $opterr=
  GetOptions('help|?' => \$help, man => \$man, 'seq=s'=>\$seqfile,
	    'output=s'=>\$output_opt);
pod2usage(1) if $help;
pod2usage(-exitstatus => 0, -verbose => 2) if $man;

pod2usage({-verbose=>1,-exitval=>2,-message=>'Need some input file...'}) if $#ARGV<0;

my $round;
foreach my $inf (@ARGV){
  my ($iprefix,$dir,$ext)=fileparse($inf,qr{\..*});
  $seqfile="$dir/$iprefix.seqs" unless $seqfile;
  die "Need $seqfile (or specify manually)" unless -f $seqfile;
  open(my $infh,$inf) or die "Couldn't open $inf";
  my @seqs=map { {file=>$_} } (split(/\n/,slurp($seqfile)));
  foreach my $seq (@seqs){
    print "Loading regions for $seq->{file}\n";
    $seq->{regions}=[getRegions($seq->{file})];
  }

  my $outf=($output_opt and $round) ? "$output_opt.$round":$output_opt;
  $outf="$inf.anchors" unless $outf;
  print STDERR "Warning: Overwriting $outf\n" if -f $outf;
  open(my $ofh,">$outf");
  while(my $line=<$infh>){
    my @anchors=getAnchors($line);
    die "Illegal number of anchors (".scalar(@anchors).")" unless scalar(@anchors)==scalar(@seqs);
    print $ofh join("\t",
		    map {fixInRegion($anchors[$_],$seqs[$_]->{regions})} 0..$#anchors)."\n";
  }
}

sub fixInRegion {
  my ($anchor,$regions)=@_;
  return (1,$anchor->{start},$anchor->{stop}) unless $regions;
  die "programming error ftl!" unless ref($regions) eq 'ARRAY';
  my $region=$$regions[$anchor->{chrom}-1];
  die "Invalid region: $anchor->{chrom}" unless $region;
  return ($anchor->{start}+$region->{start},$anchor->{stop}+$region->{start},$anchor->{sign});
}

sub anchorToString {
  my ($anchor)=@_;
  return join("\t",$anchor->{start},$anchor->{stop},$anchor->{sign});
}

sub getAnchors {
  my ($line)=@_;
  my @d=split(/\t| /,$line);
  my @anchors;
  while(@d){
    my ($chrom,$start,$stop,$sign)=map {shift(@d)} (1..4);
    ($start,$stop)=(-$start,-$stop) if($start<0);
    ($start,$stop)=($stop,$start) if $stop<$start; #be flexible
    push(@anchors,{chrom=>$chrom,start=>$start,stop=>$stop,sign=>$sign});
  }
  return @anchors
}

sub getRegions {
  my ($file)=@_;
  die "No file specified?" unless $file;
  return undef unless $file=~m/\.stitch$/;
  open(my $fh,$file) or die "Couldn't open stitch sub-file $file";
  my @res;
  while(<$fh>){
    my @dat=split(/\t/);
    push(@res,{file=>$dat[0],length=>$dat[1],start=>$dat[2],stop=>$dat[3]});
  }
  return @res;
}

sub slurp {
  local $/;
  open(my $fh,"@_") or return;
  return <$fh>;
}
