/*
Murasaki - multiple genome global alignment program
Copyright (C) 2006-2007 Keio University
(Kris Popendorf) <comp@bio.keio.ac.jp> (2006)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/////////////////////
// murasaki config options manager
////////////////////

#ifndef __OPTIONS_H
#define __OPTIONS_H

#include "sequence.h"
#include <string>
#include <iostream>

using namespace std;

enum HashType {ArrayHash_t, MSetHash_t, MySQLHash_t, SQLiteHash_t, EcoHash_t,Last_t};
extern int mpi_id;

class Options {
 public:
  //config options
  word verbose;
  int quick_hash;
  word hitfilter;
  int histogram;
  bool user_seed;
  int rand_seed;
  bool repeatMask;
  bool skip1to1,skipFwd,skipRev;
  bool doubleHash;
  bool hashOnly;
  bool hashCache;
  bool bitscore;
  HashType hashPref;
  string alignment_name;
  string output_dir;
  string config_file;
  string cache_dir;
  
  word seedfilter,hashfilter,mergeFilter;

  bool joinNeighbors;
  float joinSpec;
  int joinDist;

  int hashSkip;

  bool auto_hashbits;
  bool dumpRegions,dumpEntropy,leaveRecords;

  unsigned long targetMemory;
  bool userSetMemory;
  bool retainMembers;
  bool commutative;
  bool useHumanTime;
  bool repeatMap;
  word anchorProgressCheck;
  bool extractInOrder,tfidf,reverseOtf;
  word rifts;


  int mpi_hashers;
  bool mpi_noCake;
  bool mpi_fileDistro;
  bool mpi_anchorInSpareTime;
  word mpi_maxBuffers;
  bool mpi_bigFirst,mpi_hostbalance,mpi_memoryBalance,mpi_distMerge,mpi_distCollect;
  
  bool use_shm_mmap,mmap_writePerHost;
  //mmap has a potential performance hit because it requires sequence distribution via nfs/disk,
  //so we leave it as an option. sysv requires no disk backing, so if we have it, always use it.
  bool use_shm_sysv;

  //computed dynamically in solidfy()
  string prefix;
  string hashStatus_record;
  string hashHisto_record;
  string hashDetailed_record;
  string seq_record;
  string anchor_record;
  string repeat_record;
  string pat_record;
  string region_record;
  string status_record;
  string options_record;
  string entropy_record;
  string anchorProgress_record;

  int seqreadOptions;

  //functions
  Options();
  void preInit();
  void loadConfig(string config);
  void solidify();
  void commandline_opts(int argc,char **argv);
  string hashPref_s() const;
  friend ostream& operator<<(ostream& os,const Options &a);
};

string program_help();
bool isYes(const char* str);
bool isNo(const char* str);
void parseYesNo(const char* str,bool& toggle,const char *name);

#endif
